using System;
using System.Collections;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Net;
using System.Windows.Forms;
using Microsoft.Practices.EnterpriseLibrary.Common;
using Microsoft.Practices.EnterpriseLibrary.Configuration;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using FlickrNet;
using System.Configuration;

namespace FlickrExplorer
{
	public class SendInfosEventArgs: EventArgs 
	{
		public SendInfosEventArgs(string comment) 
		{
			this.comment = comment;
		}

        public SendInfosEventArgs(string comment, int current, int total)
        {
            this.comment = comment;
            this.current = current;
            this.total = total;
        }

		public string comment;
        public int current = 0;
        public int total = 0;

	}

	public class FileAlreadyExistsEventArgs : EventArgs
	{
		public FileAlreadyExistsEventArgs(string FileName, string NewFileName)
		{
			this.FileName = FileName;
            this.NewFileName = NewFileName;
		}

		public string FileName;
        public string NewFileName;
	}

	public delegate void SendInfosEventHandler(Object sender, SendInfosEventArgs e);
	public delegate void FileAlreadyExistsEventHandler(Object sender, FileAlreadyExistsEventArgs e);

	/// <summary>
	/// Description rsume de Process.
	/// </summary>
	public class Process
	{
		public Flickr flickr;
		public bool anonymous;
        //public string userName;
		public configuration cfg;
		public string photoSetId;
		private string saveAs_PhotoTitle;
		private string saveAs_PhotoNumber;
		private string saveAs_PhotoCount;
		public bool AlwaysOverwrite = false;
		public bool NeverOverwrite = false;
		public bool Overwrite = false;
        public bool AutomaticallyRename = false;
        public bool AlwaysAutomaticallyRename = false;
        public bool NeverAutomaticallyRename = false;
        public FlickrNet.Auth token = null;
        private bool stopProcess = false;

        public const string SharedSecret = "872b6e0cbb53f55a";

        int PhotosPerPage = 500;

        private string[] _sizesPossible = { "Original", "Large", "Medium", "Small", "Thumbnail" };

		WebProxy proxy;

		public event SendInfosEventHandler SendInfos;
		public event FileAlreadyExistsEventHandler FileAlreadyExists;

		private void Log(string comment)
		{
			if (SendInfos != null)
				SendInfos(this, new SendInfosEventArgs(comment));
		}

        private void Log(string comment, int current, int total)
        {
            if (SendInfos != null)
                SendInfos(this, new SendInfosEventArgs(comment, current, total));
        }

        private string Log(Exception ex)
        {
            string msg;
            if (System.Configuration.ConfigurationSettings.AppSettings["debug"] != "0")
            {
                msg = ex.ToString();
            }
            else
            {
                msg = "Error : "+ex.Message;
            }
            if (SendInfos != null)
                SendInfos(this, new SendInfosEventArgs(msg));
            return msg;
        }

        public void StopProcess()
        {
            stopProcess = true;
        }

		public Process(bool anonymousLogin)
		{
			try
			{
				cfg = ConfigurationManager.GetConfiguration(configuration.FlickrExplorerSettings) as configuration;
                cfg.AuthToken.Token = null;
                anonymous = anonymousLogin;

				Log("Connection in progress ...");

                Login();
				flickr.DownloadingPicture += new FlickrNet.DownloadingPictureEventHandler(DownloadingPicture);
			}
			catch (Exception ex)
			{
				if (ExceptionPolicy.HandleException(ex,"General"))
				{
                    MessageBox.Show(Log(ex));
				}
			}
		}

        private void Login()
        {
            if (anonymous)
            {
                flickr = new Flickr(cfg.APIKey);
                SetProxy();
            }
            else
                if (cfg.AuthToken.Token == null)
                {
                    //Getting Flickr Authentication token                    
                    flickr = new Flickr(cfg.APIKey, SharedSecret);
                    SetProxy();

                    string Frob = flickr.AuthGetFrob();
                    string url = flickr.AuthCalcUrl(Frob, AuthLevel.Read);
                    System.Diagnostics.Process.Start(url);
                    System.Windows.Forms.MessageBox.Show("The Flickr website is opened to authenticate, click here when it's done.");
                    token = flickr.AuthGetToken(Frob);

                    //Storing Token                   
                    cfg.AuthToken = token;
                    flickr.AuthToken = token.Token;
                }
                else
                {
                    token = cfg.AuthToken;
                    flickr = new Flickr(cfg.APIKey, SharedSecret.ToString(), token.Token);
                    SetProxy();
                }
                        
        }

        private void SetProxy()
        {
            if (cfg.ProxyAddress.Length > 0)
            {
                proxy = new WebProxy(cfg.ProxyAddress);
                if (cfg.ProxyAuthenticationRequired)
                {
                    NetworkCredential nc = new NetworkCredential(cfg.ProxyUser, cfg.ProxyPassword);
                    proxy.Credentials = nc;
                }
                flickr.Proxy = proxy;
            }
        }

		private void DownloadingPicture(object sender, DownloadingPictureEventArgs e)
		{
            float c = int.Parse(saveAs_PhotoNumber);
            float t = int.Parse(saveAs_PhotoCount);
            int current = (int)Math.Round((c / t) * 100, 0);
            Log("Loading : " + saveAs_PhotoTitle + " (" + saveAs_PhotoNumber + "/" + saveAs_PhotoCount + ") -> " + e.BytesRead.ToString() + " bytes read.", current, 100);
		}

		

		public void reloadConfig()
		{
			try
			{
				Log("Reloading configuration ...");
				cfg = ConfigurationManager.GetConfiguration(configuration.FlickrExplorerSettings) as configuration;
				Log("Configuration reloaded ...");
			}
			catch (Exception ex)
			{
				if (ExceptionPolicy.HandleException(ex,"General"))
				{
                    MessageBox.Show(Log(ex));
				}
			}
		}

		public void populateTreeView(string userName, ref TreeView Arborescence)
		{
			try
			{
				Log("Loading arborescence ...");
				TreeNode nodeUser = new TreeNode("User : "+userName);
                nodeUser.ImageIndex = 2;
                nodeUser.SelectedImageIndex = 2;
                nodeUser.StateImageIndex = 2;
                nodeUser.Tag = userName;
				Arborescence.Nodes.Add(nodeUser);
			
				TreeNode nodePhotoSets = new TreeNode("Photosets");
                nodePhotoSets.ImageIndex = 0;
                nodePhotoSets.SelectedImageIndex = 0;
                nodePhotoSets.StateImageIndex = 0;
				nodeUser.Nodes.Add(nodePhotoSets);

                TreeNode nodePhotos = new TreeNode("Photo Stream");
                nodePhotos.ImageIndex = 0;
                nodePhotos.SelectedImageIndex = 0;
                nodePhotos.StateImageIndex = 0;
                nodeUser.Nodes.Add(nodePhotos);

				Application.DoEvents();

                FoundUser user = flickr.PeopleFindByUsername(userName);
				Application.DoEvents();
				Photosets photosets = flickr.PhotosetsGetList(user.UserId);
                if (photosets.PhotosetCollection != null)
				    foreach (Photoset photoset in photosets.PhotosetCollection)
				    {
					    Application.DoEvents();
					    TreeNode nodePhotoSet = new TreeNode(photoset.Title);
					    nodePhotoSet.Tag = photoset.PhotosetId+"|"+photoset.NumberOfPhotos;
                        nodePhotoSet.ImageIndex = 1;
                        nodePhotoSet.SelectedImageIndex = 1;
                        nodePhotoSet.StateImageIndex = 1;
					    nodePhotoSets.Nodes.Add(nodePhotoSet);
				    }
				Log("Arborescence loaded.");
			}
			catch (Exception ex)
			{
                if (ExceptionPolicy.HandleException(ex, "General"))
                {
                    MessageBox.Show(Log(ex));
                }
			}
		}

		public void populatePhotosList(ref ListView PhotosList, string photosetId, int numberOfPhotos)
		{
            try
            {
                Log("Opening photoset ...");
                ImageList img = PhotosList.SmallImageList;
                PhotosList.Clear();
                img.Images.Clear();
                this.photoSetId = photosetId;

                int j = 0;
                int k = 0;
                
                Double NumberOfPages = Math.Ceiling((double)(numberOfPhotos / PhotosPerPage))+1;

                for (int p = 1; p <= NumberOfPages; p++)
                {
                    Photoset photoset = flickr.PhotosetsGetPhotos(photosetId, p, PhotosPerPage);
                    foreach (Photo photo in photoset.PhotoCollection)
                    {
                        j++;
                        Log("Loading photo name : Page " + p.ToString() + "/" + NumberOfPages.ToString() + ", Photo " + j.ToString() + "/" + numberOfPhotos.ToString());
                        ListViewItem item = new ListViewItem();
                        item.Tag = photo.PhotoId;
                        PhotosList.Items.Add(item);
                        Application.DoEvents();
                    }
                    saveAs_PhotoCount = PhotosList.Items.Count.ToString();
                    for (int i = k; i < PhotosList.Items.Count; i++)
                    {
                        saveAs_PhotoNumber = ((int)(i + 1)).ToString();
                        Log("Loading photo thumbnail : Page " + p.ToString() + "/" + NumberOfPages.ToString() + ", Photo " + saveAs_PhotoNumber + "/" + numberOfPhotos);
                        Photo photo = photoset.PhotoCollection[i-((p-1)*PhotosPerPage)];
                        if (photo.PhotoId == (string)PhotosList.Items[i].Tag)
                        {
                            try
                            {
                                img.Images.Add(Image.FromStream(flickr.DownloadPicture(photo.ThumbnailUrl)));
                                PhotosList.Items[i].ImageIndex = i;
                                Application.DoEvents();
                            }
                            catch (ArgumentOutOfRangeException)
                            {
                                break;
                            }
                            catch (FlickrException f)
                            {
                                //if (f.Code == 301)
                                //{
                                //    break;
                                //}
                                //else
                                //{
                                throw;
                                //}
                            }
                        }
                        Application.DoEvents();
                    }
                    k = j+1;
                }
            }
            catch (IndexOutOfRangeException)
            {
            }
            catch (Exception ex)
            {
                if (ExceptionPolicy.HandleException(ex, "General"))
                {
                    MessageBox.Show(Log(ex));
                }
            }
        
		}

        public void populatePhotoStream(ref ListView PhotosList, string UserId)
        {
            try
            {
                Log("Opening Photo Stream ...");
                ImageList img = PhotosList.SmallImageList;
                PhotosList.Clear();
                img.Images.Clear();

                FoundUser user = flickr.PeopleFindByUsername(UserId);
                Photos photoset = flickr.PhotosSearch(user.UserId, new string[] { });

                int i = 0;
                foreach (Photo photo in photoset.PhotoCollection)
                {
                    i++;
                    Log("Loading photo name : " + i.ToString() + "/" + photoset.PhotoCollection.Length.ToString());
                    ListViewItem item = new ListViewItem();
                    item.Tag = photo.PhotoId;
                    PhotosList.Items.Add(item);
                    Application.DoEvents();
                }
                saveAs_PhotoCount = PhotosList.Items.Count.ToString();
                for (i = 0; i < PhotosList.Items.Count; i++)
                {
                    saveAs_PhotoNumber = ((int)(i + 1)).ToString();
                    Log("Loading photo thumbnail : " + saveAs_PhotoNumber + "/" + saveAs_PhotoCount);
                    Photo photo = photoset.PhotoCollection[i];
                    if (photo.PhotoId == (string)PhotosList.Items[i].Tag)
                    {
                        try
                        {
                            img.Images.Add(Image.FromStream(flickr.DownloadPicture(photo.ThumbnailUrl)));
                            PhotosList.Items[i].ImageIndex = i;
                        }
                        catch (ArgumentOutOfRangeException)
                        {
                            break;
                        }
                        catch (FlickrException f)
                        {
                            //if (f.Code == 301)
                            //{
                            //    break;
                            //}
                            //else
                            //{
                                throw;
                            //}
                        }
                    }
                    Application.DoEvents();
                }
            }
            catch (IndexOutOfRangeException)
            {
            }
            catch (Exception ex)
            {
                if (ExceptionPolicy.HandleException(ex, "General"))
                {
                    MessageBox.Show(Log(ex));
                }
            }
        }

		public void previewPhoto(ref PictureBox photoPreview, string photoId, ref string desc, ref string meta, int numberOfPhotos)
		{
			meta = "";
			desc = "";
			Photo photo = null;

            Log("Looking for the photo information ...");

            Double NumberOfPages = Math.Ceiling((double)(numberOfPhotos / PhotosPerPage));

            for (int p = 1; p <= NumberOfPages; p++)
            {
                Photo[] photos = flickr.PhotosetsGetPhotos(photoSetId, p, PhotosPerPage).PhotoCollection;
                foreach (Photo pho in photos)
                {
                    if (pho.PhotoId == photoId)
                    {
                        photo = pho;
                        break;
                    }
                }
                if (photo != null)
                {
                    Log("Loading : " + photo.Title + " ...");
                    photoPreview.Image = System.Drawing.Image.FromStream(flickr.DownloadPicture(photo.SmallUrl));
                    PhotoInfo infos = flickr.PhotosGetInfo(photo.PhotoId);
                    desc = "Title : " + photo.Title + "\r\nDescription : " + infos.Description;
                    meta = "Title : " + photo.Title;
                    meta += "\r\nOwner : " + photo.OwnerName;
                    meta += "\r\nDescription : " + infos.Description;
                    meta += "\r\nDate taken : " + photo.DateTaken;
                    meta += "\r\nDate uploaded : " + photo.DateUploaded;
                    if (infos.Tags != null)
                    {
                        meta += "\r\n";
                        meta += "\r\nTags : ";
                        try
                        {
                            foreach (PhotoInfoTag tag in infos.Tags.TagCollection)
                                meta += "\r\n  " + tag.TagText;
                        }
                        catch
                        { }
                    }
                    meta += "\r\n";
                    meta += "\r\nLicense : " + photo.License;
                    meta += "\r\nPublic : " + photo.IsPublic;
                    meta += "\r\nFriend : " + photo.IsFriend;
                    meta += "\r\nFamily : " + photo.IsFamily;
                    meta += "\r\nComments count : " + infos.CommentsCount;
                    if (infos.Notes.NoteCollection != null)
                    {
                        meta += "\r\n";
                        meta += "\r\nNotes : ";
                        foreach (PhotoInfoNote note in infos.Notes.NoteCollection)
                        {
                            meta += "\r\n" + note.AuthorName + " wrote : " + note.NoteText;
                        }
                    }
                    if (infos.ExifTagCollection != null)
                    {
                        meta += "\r\n";
                        meta += "\r\nTags EXIF : ";
                        foreach (ExifTag tag in infos.ExifTagCollection)
                        {
                            meta += "\r\n" + tag.Label + " : " + tag.Tag;
                        }
                    }
                    Log("");
                    break;
                }
            }

		}

		public void saveAs(string selectedPath, ListView PhotosList)
		{
			string FilePath = String.Empty;
            string photoUrl = string.Empty;
            string ext = ".jpg";
            ArrayList processedPhotoID = new ArrayList();

            float t = PhotosList.CheckedItems.Count;
            Double NumberOfPages = Math.Ceiling((double)(t / PhotosPerPage));

            for (int p = 1; p <= NumberOfPages; p++)
            {
                Log(string.Empty, 1, 100);
                Photo[] photos = flickr.PhotosetsGetPhotos(photoSetId, p, PhotosPerPage).PhotoCollection;

                int i = 0;
                saveAs_PhotoCount = PhotosList.CheckedItems.Count.ToString();

                float c = 0;
                foreach (ListViewItem item in PhotosList.CheckedItems)
                {
                    Application.DoEvents();
                    if (stopProcess) break;
                    string photoId = (string)item.Tag;
                    foreach (Photo photo in photos)
                    {
                        Application.DoEvents();
                        if (photo.PhotoId == photoId)
                        {
                            try
                            {
                                i++;
                                saveAs_PhotoTitle = photo.Title;
                                saveAs_PhotoNumber = i.ToString();
                                c = i;
                                int current = (int)Math.Round((c / t) * 100, 0);
                                Log("Saving photo : page " + p.ToString() + "/" + NumberOfPages.ToString() + ", " + photo.Title + " (" + i.ToString() + "/" + PhotosList.CheckedItems.Count + ") ...", current, 100);
                                Application.DoEvents();

                                FilePath = selectedPath + "\\" + photo.Title + ext;
                                if (File.Exists(FilePath))
                                {
                                    int suffix = 2;
                                    string newFilePath = FilePath.Replace(ext, "-" + suffix.ToString() + ext);

                                    while (File.Exists(newFilePath))
                                    {
                                        suffix++;
                                        newFilePath = FilePath.Replace(ext, "-" + suffix.ToString() + ext);
                                    }
                                    string newFileName = Path.GetFileName(newFilePath);
                                    FileAlreadyExists(this, new FileAlreadyExistsEventArgs(FilePath, newFileName));
                                    if (!Overwrite && !processedPhotoID.Contains(photo.PhotoId)
                                        && (AlwaysAutomaticallyRename || AutomaticallyRename))
                                    {
                                        FilePath = newFilePath;
                                    }
                                }
                                if (Overwrite || !File.Exists(FilePath))
                                {
                                    // The label for the size, such as "Thumbnail", "Small", 
                                    // "Medium", "Large" and "Original".
                                    Sizes ss = flickr.PhotosGetSizes(photo.PhotoId);
                                    bool ok = false;
                                    foreach (string sizePossible in _sizesPossible)
                                    {
                                        foreach (FlickrNet.Size s in ss.SizeCollection)
                                        {
                                            if (string.Compare(sizePossible, s.Label, true) == 0)
                                            {
                                                photoUrl = s.Source;
                                                ok = true;
                                                break;
                                            }
                                            if (ok) break;
                                        }
                                    }

                                    //Image img = null;
                                    Stream str = null;
                                    try
                                    {
                                        str = flickr.DownloadPicture(photoUrl);
                                        //img = Image.FromStream(str);
                                    }
                                    catch (ArgumentException)
                                    {
                                        Flickr.FlushCache();
                                        str = flickr.DownloadPicture(photoUrl);
                                        //img = Image.FromStream(str);
                                    }
                                    FilePath = FilePath.Replace("\"", "'");
                                    StreamWriter swr = new StreamWriter(FilePath);
                                    BinaryReader brd = new BinaryReader(str);
                                    BinaryWriter bwr = new BinaryWriter(swr.BaseStream);

                                    byte[] val;
                                    val = brd.ReadBytes(Convert.ToInt32(str.Length));
                                    bwr.Write(val);
                                    bwr.Close();
                                    brd.Close();
                                    str.Close();

                                    //img.Save(FilePath);
                                    if (!AlwaysOverwrite)
                                        Overwrite = false;

                                    item.Checked = false;
                                    processedPhotoID.Add(photo.PhotoId);
                                    Application.DoEvents();

                                    if (stopProcess) break;
                                }
                            }
                            catch (Exception ex)
                            {
                                if (ExceptionPolicy.HandleException(ex, "General"))
                                {
                                    MessageBox.Show(Log(ex));
                                }
                            }
                        }
                    }
                }
            }

            stopProcess = false;
		}

	}
}
